using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using CommonHelpers.Common;
using Telerik.XamarinForms.DataControls.TreeView;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;

[assembly: XamlCompilation(XamlCompilationOptions.Compile)]
namespace ItemStyleDemo.Portable
{
    public partial class MainPage : ContentPage
    {
        public MainPage()
        {
            InitializeComponent();
        }
        
        private void MyTreeView_OnItemTapped(object sender, TreeViewItemEventArgs e)
        {
            (e.Item as MyDataItem).IsHighlighted = !(e.Item as MyDataItem).IsHighlighted;
        }
    }
    
    public class BoolToColorConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (value is bool b)
            {
                return b ? Color.Blue : Color.Black;
            }

            return Color.Green;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class ViewModel : ViewModelBase
    {
        public ViewModel()
        {
            this.SmallSource = new ObservableCollection<MyDataItem>();
            this.LoadData();
        }
        
        public ObservableCollection<MyDataItem> SmallSource { get; set; }

        private void LoadData()
        {
            this.SmallSource.Add(new MyDataItem
            {
                Name = "Item 1",
                Children = new List<MyDataItem>
                {
                        new MyDataItem
                        {
                            Name = "Item 11",
                            Children = new List<MyDataItem>
                            {
                                new MyDataItem
                                {
                                    Name = "Item 111",
                                },
                                new MyDataItem
                                {
                                    Name = "Item 112"
                                },
                                new MyDataItem
                                {
                                    Name = "Item 113"
                                }
                            }
                        },
                        new MyDataItem
                        {
                            Name = "Item 12"
                        }
                    }
            });
            this.SmallSource.Add(new MyDataItem
            {
                Name = "Item 2",
                Children = new List<MyDataItem>
                {
                        new MyDataItem
                        {
                            Name = "Item 21",
                            Children = new List<MyDataItem>
                            {
                                new MyDataItem
                                {
                                    Name = "Item 211",
                                },
                                new MyDataItem
                                {
                                    Name = "Item 212"
                                },
                                new MyDataItem
                                {
                                    Name = "Item 213"
                                }
                            }
                        },
                        new MyDataItem
                        {
                            Name = "Item 22",
                            Children = new List<MyDataItem>
                            {
                                new MyDataItem
                                {
                                    Name = "Item 221",
                                },
                                new MyDataItem
                                {
                                    Name = "Item 222"
                                },
                                new MyDataItem
                                {
                                    Name = "Item 223"
                                }
                            }
                        }
                    }
            });
        }
    }
    
    public class MyDataItem : BindableBase
    {
        private IList<MyDataItem> _children;
        private string _name;
        private bool _isHighlighted;

        public MyDataItem()
        {
            this.Children = new ObservableCollection<MyDataItem>();
        }

        public IList<MyDataItem> Children
        {
            get => _children;
            set => SetProperty(ref _children, value);
        }

        public string Name
        {
            get => this._name;
            set => SetProperty(ref _name, value);
        }

        public bool IsHighlighted
        {
            get => _isHighlighted;
            set => SetProperty(ref _isHighlighted, value);
        }
        
    }

    // This is to indent the items properly
    public class LevelToMarginConverter : IValueConverter
    {
        public object Convert(object value, Type type, object parameter, CultureInfo culture)
        {
            var level = (uint)value;

            return new Thickness(level * 40, 0, 0, 0);
        }

        public object ConvertBack(object value, Type type, object parameter, CultureInfo culture)
        {
            throw new NotSupportedException();
        }
    }
}
